@@ -16,13 +16,31 @@
 #ifndef QGSTEST_H
 #define QGSTEST_H
 
-#include <QtTest/QtTest>
-#include "qgsrectangle.h"
+#include <QtTest/QTest>
 #include "qgsapplication.h"
 
+#include "qgsabstractgeometry.h"
+#include "qgscurve.h"
+#include "qgscircularstring.h"
+#include "qgscompoundcurve.h"
+#include "qgslinestring.h"
+#include "qgsgeometrycollection.h"
+#include "qgsmulticurve.h"
+#include "qgsmultilinestring.h"
+#include "qgsmultipoint.h"
+#include "qgsmultisurface.h"
+#include "qgsmultipolygon.h"
+#include "qgspoint.h"
+#include "qgssurface.h"
+#include "qgscurvepolygon.h"
+#include "qgspolygon.h"
+#include "qgstriangle.h"
+#include "qgsrectangle.h"
+#include "qgsregularpolygon.h"
+
+
 #define QGSTEST_MAIN(TestObject) \
   QT_BEGIN_NAMESPACE \
-  QTEST_ADD_GPU_BLACKLIST_SUPPORT_DEFS \
   QT_END_NAMESPACE \
   int main(int argc, char *argv[]) \
   { \
@@ -30,7 +48,6 @@
     app.init(); \
     app.setAttribute(Qt::AA_Use96Dpi, true); \
     QTEST_DISABLE_KEYPAD_NAVIGATION \
-    QTEST_ADD_GPU_BLACKLIST_SUPPORT \
     TestObject tc; \
     QTEST_SET_MAIN_SOURCE_PATH \
     return QTest::qExec(&tc, argc, argv); \
@@ -41,7 +58,7 @@
     bool _xxxresult = qgsDoubleNear( value, expected, epsilon ); \
     if ( !_xxxresult  ) \
     { \
-      qDebug( "Expecting %f got %f (diff %f > %f)", static_cast< double >( expected ), static_cast< double >( value ), std::fabs( static_cast< double >( expected ) - value ), static_cast< double >( epsilon ) ); \
+      qDebug( "Expecting %.10f got %.10f (diff %.10f > %.10f)", static_cast< double >( expected ), static_cast< double >( value ), std::fabs( static_cast< double >( expected ) - value ), static_cast< double >( epsilon ) ); \
     } \
     QVERIFY( qgsDoubleNear( value, expected, epsilon ) ); \
   }(void)(0)
@@ -58,19 +75,19 @@
 #define QGSCOMPARENEARPOINT(point1,point2,epsilon) { \
     QGSCOMPARENEAR( point1.x(), point2.x(), epsilon ); \
     QGSCOMPARENEAR( point1.y(), point2.y(), epsilon ); \
-  }
+  }(void)(0)
 
 #define QGSCOMPARENEARRECTANGLE(rectangle1,rectangle2,epsilon) { \
     QGSCOMPARENEAR( rectangle1.xMinimum(), rectangle2.xMinimum(), epsilon ); \
     QGSCOMPARENEAR( rectangle1.xMaximum(), rectangle2.xMaximum(), epsilon ); \
     QGSCOMPARENEAR( rectangle1.yMinimum(), rectangle2.yMinimum(), epsilon ); \
     QGSCOMPARENEAR( rectangle1.yMaximum(), rectangle2.yMaximum(), epsilon ); \
-  }
+  }(void)(0)
 
 //sometimes GML attributes are in a different order - but that's ok
 #define QGSCOMPAREGML(result,expected) { \
     QCOMPARE( result.replace( QStringLiteral("ts=\" \" cs=\",\""), QStringLiteral("cs=\",\" ts=\" \"") ), expected ); \
-  }
+  }(void)(0)
 
 /**
  * QGIS unit test utilities.
@@ -92,12 +109,111 @@ namespace QgsTest
 }
 
 /**
- * Formatting QgsRectangle for QCOMPARE pretty printing
+ * For QCOMPARE pretty printing
  */
-char *toString( const QgsRectangle &r )
+char *toString( const QgsAbstractGeometry &geom )
 {
-  return QTest::toString( QStringLiteral( "QgsRectangle(%1, %2, %3, %4)" ).arg( QString::number( r.xMinimum() ), QString::number( r.yMinimum() ), QString::number( r.xMaximum() ), QString::number( r.yMaximum() ) ) );
+  return QTest::toString( geom.asWkt() );
 }
 
+char *toString( const QgsCurve &geom )
+{
+  return QTest::toString( geom.asWkt() );
+}
+
+char *toString( const QgsCircularString &geom )
+{
+  return QTest::toString( geom.asWkt() );
+}
+
+char *toString( const QgsCompoundCurve &geom )
+{
+  return QTest::toString( geom.asWkt() );
+}
+
+char *toString( const QgsLineString &geom )
+{
+  return QTest::toString( geom.asWkt() );
+}
+
+char *toString( const QgsGeometryCollection &geom )
+{
+  return QTest::toString( geom.asWkt() );
+}
+
+char *toString( const QgsMultiCurve &geom )
+{
+  return QTest::toString( geom.asWkt() );
+}
+
+char *toString( const QgsMultiLineString &geom )
+{
+  return QTest::toString( geom.asWkt() );
+}
+
+char *toString( const QgsMultiPoint &geom )
+{
+  return QTest::toString( geom.asWkt() );
+}
+
+char *toString( const QgsMultiSurface &geom )
+{
+  return QTest::toString( geom.asWkt() );
+}
+
+char *toString( const QgsMultiPolygon &geom )
+{
+  return QTest::toString( geom.asWkt() );
+}
+
+char *toString( const QgsPoint &geom )
+{
+  return QTest::toString( geom.asWkt() );
+}
+
+char *toString( const QgsPointXY &geom )
+{
+  return QTest::toString( geom.asWkt() );
+}
+
+char *toString( const QgsSurface &geom )
+{
+  return QTest::toString( geom.asWkt() );
+}
+
+char *toString( const QgsCurvePolygon &geom )
+{
+  return QTest::toString( geom.asWkt() );
+}
+
+char *toString( const QgsPolygon &geom )
+{
+  return QTest::toString( geom.asWkt() );
+}
+
+char *toString( const QgsRegularPolygon &geom )
+{
+  return QTest::toString( geom.toString() );
+}
+
+char *toString( const QgsTriangle &geom )
+{
+  return QTest::toString( geom.asWkt() );
+}
+
+char *toString( const QgsRectangle &geom )
+{
+  return QTest::toString( geom.toString() );
+}
+
+char *toString( const QgsEllipse &geom )
+{
+  return QTest::toString( geom.toString() );
+}
+
+char *toString( const QgsCircle &geom )
+{
+  return QTest::toString( geom.toString() );
+}
 
 #endif // QGSTEST_H
